  (**************************************************************************
            Author: angshu
                    angshu@agnisoft.com
            Copyright (C) 2000 Agni Software Pvt. Ltd.
            All Rights Reserved.
            http://www.agnisoft.com

    TDecisionGridRpt prints a DecisionGrid.
    Call to TDecisionGridRpt.ShowPreview procedure brings up the QuickReport
    Preview or a Custom preview if the TDecisionGridRpt.QuickRep property
    is set. TDecisionGridRpt.OutOfPageCaution property is to be set in code.
  ***************************************************************************)

unit agDecisionGridRpt;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  QuickRpt, ExtCtrls, mxgrid, Qrctrls, QrPrntr, QRExport, db;

type
  TDecisionGridRpt = Class(TComponent)
  private
    m_nColCount,
     m_nFixedCols,
     m_nRowCount,
     m_nFixedRows,
     m_nCurrentRow,
     m_nPrintableCols 	: Integer;

    m_bOutOfPageCaution,
     m_bCreateQR,
     m_bBandCreated 	: Boolean;

    m_QRDataset    	: TDataSet;
    m_QRDetailBand 	: TQRCustomBand;

    m_CaptionColor,
     m_LabelColor,
     m_DataColor,
     m_DataSumColor,
     m_LabelSumColor,
     m_GridColor    	: TColor;

    m_CaptionFont,
     m_LabelFont,
     m_DataFont     	: TFont;

    m_bPrintInColor,
     m_bShowGridLines 	: Boolean;

    m_nColumnWidth,
     m_nRowHeight     	: Integer;
    m_LabelList 	   	: TList;
    m_DCGrid  			: TDecisionGrid;
    m_QR      			: TQuickRep;

    FQRBeforePrintEvent: TQRReportBeforePrintEvent;

    procedure SetQuickRep(Value: TQuickRep);
    procedure SetDCGrid(const Value: TDecisionGrid);
    procedure SetPrintInColor(const Value: Boolean);
    procedure SetShowGridLines(const Value: Boolean);
    procedure SetCaptionColor(const Value: TColor);
    procedure SetCaptionFont(const Value: TFont);
    procedure SetLabelColor(const Value: TColor);
    procedure SetLabelFont(const Value: TFont);
    procedure SetDataColor(const Value: TColor);
    procedure SetDataFont(const Value: TFont);
    procedure SetColumnWidth(const Value: Integer);
    procedure SetRowHeight(const Value: Integer);
    procedure SetDataSumColor(const Value: TColor);
    procedure SetLabelSumColor(const Value: TColor);
    procedure SetGridColor(const Value: TColor);

    procedure CreateLabels;
    procedure DeleteLabels;
    procedure m_QRBeforePrint(Sender: TCustomQuickRep; var PrintReport: Boolean);
    procedure m_QRNeedData(Sender: TObject; var MoreData: Boolean);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    property OutOfPageCaution : boolean read m_bOutOfPageCaution write m_bOutOfPageCaution;
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure ShowPreview;
  published
    property DecisionGrid : TDecisionGrid read  m_DCGrid write SetDCGrid;
    property QuickRep     : TQuickRep read m_QR write SetQuickRep default nil;

    property PrintInColor : Boolean   read m_bPrintInColor  write SetPrintInColor  default True;
    property ShowGridLines: Boolean   read m_bShowGridLines write SetShowGridLines default True;
    property ColumnWidth  : Integer   read m_nColumnWidth   write SetColumnWidth   default 100;
    property RowHeight    : Integer   read m_nRowHeight     write SetRowHeight     default 20;

    property CaptionFont  : TFont  read m_CaptionFont   write SetCaptionFont;
    property LabelFont    : TFont  read m_LabelFont     write SetLabelFont;
    property DataFont	   : TFont  read m_DataFont		 write SetDataFont;

    property CaptionColor : TColor read m_CaptionColor  write SetCaptionColor;
    property LabelColor   : TColor read m_LabelColor    write SetLabelColor;
    property DataColor    : TColor read m_DataColor     write SetDataColor;
    property DataSumColor : TColor read m_DataSumColor  write SetDataSumColor;
    property LabelSumColor: TColor read m_LabelSumColor write SetLabelSumColor;
    property GridColor    : TColor read m_GridColor     write SetGridColor;
  end;

procedure Register;

implementation
{$R *.RES}

procedure Register;
begin
  RegisterComponents('Samples', [TDecisionGridRpt]);
end;


{ TDecisionGridRpt }

constructor TDecisionGridRpt.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);

    m_bOutOfPageCaution:= False;

    m_CaptionFont    	:= TFont.Create;
    m_LabelFont	  	:= TFont.Create;
    m_DataFont       	:= TFont.Create;

    m_DCGrid         	:= nil;
    m_QR	          	:= nil;
    m_bCreateQR      	:= True;
    m_QRDataset      	:= nil;
    m_QRDetailBand   	:= nil;

    m_bShowGridLines 	:= True;
    m_bPrintInColor  	:= True;
    m_nColumnWidth   	:= 100;
    m_nRowHeight     	:= 20;


    CaptionColor 	  	:= clWhite;
    LabelColor   		:= clWhite;
    DataColor    		:= clWhite;
    DataSumColor 		:= clWhite;
    LabelSumColor		:= clWhite;
    GridColor    		:= clWhite;
end;

procedure TDecisionGridRpt.SetDCGrid(const Value: TDecisionGrid);
begin
  m_DCGrid := Value;
  if Value = nil then
   begin
      CaptionColor := clWhite;
      LabelColor   := clWhite;
      DataColor    := clWhite;
      DataSumColor := clWhite;
      LabelSumColor:= clWhite;
      GridColor    := clWhite;
      Exit;
   end;

  if csLoading in ComponentState then
  	Exit;

  ColumnWidth := m_DCGrid.DefaultColWidth;
  RowHeight   := m_DCGrid.DefaultRowHeight;

  CaptionFont := m_DCGrid.CaptionFont;
  LabelFont   := m_DCGrid.LabelFont;
  DataFont    := m_DCGrid.DataFont;

  CaptionColor := m_DCGrid.CaptionColor;
  LabelColor   := m_DCGrid.LabelColor;
  DataColor    := m_DCGrid.DataColor;
  DataSumColor := m_DCGrid.DataSumColor;
  LabelSumColor:= m_DCGrid.LabelSumColor;
  GridColor    := m_DCGrid.Color;
end;

procedure TDecisionGridRpt.SetGridColor(const Value: TColor);
begin
  m_GridColor := Value;
end;

procedure TDecisionGridRpt.SetLabelColor(const Value: TColor);
begin
  m_LabelColor := Value;
end;

procedure TDecisionGridRpt.SetLabelFont(const Value: TFont);
begin
  m_LabelFont.Assign(Value);
end;

procedure TDecisionGridRpt.SetLabelSumColor(const Value: TColor);
begin
  m_LabelSumColor := Value;
end;

procedure TDecisionGridRpt.SetPrintInColor(const Value: Boolean);
begin
  m_bPrintInColor := Value;
end;

procedure TDecisionGridRpt.SetQuickRep(Value: TQuickRep);
begin
  m_QR        := Value;
  if Value <> nil then
      m_bCreateQR := False
  else
    begin
      m_bCreateQR := True;
   end;
end;

procedure TDecisionGridRpt.SetRowHeight(const Value: Integer);
begin
   m_nRowHeight := Value;
end;

procedure TDecisionGridRpt.SetShowGridLines(const Value: Boolean);
begin
  m_bShowGridLines := Value;
end;

procedure TDecisionGridRpt.SetCaptionColor(const Value: TColor);
begin
    m_CaptionColor := Value;
end;

procedure TDecisionGridRpt.SetCaptionFont(const Value: TFont);
begin
  m_CaptionFont.Assign(Value);
end;

procedure TDecisionGridRpt.SetColumnWidth(const Value: Integer);
begin
  m_nColumnWidth := Value;
end;

procedure TDecisionGridRpt.SetDataColor(const Value: TColor);
begin
  m_DataColor := Value;
end;

procedure TDecisionGridRpt.SetDataFont(const Value: TFont);
begin
  m_DataFont.Assign(Value);
end;

procedure TDecisionGridRpt.SetDataSumColor(const Value: TColor);
begin
  m_DataSumColor := Value;
end;

destructor TDecisionGridRpt.Destroy;
begin
    m_CaptionFont.Free;
    m_LabelFont.Free;
    m_DataFont.Free;
    inherited destroy;
end;

procedure TDecisionGridRpt.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
    if AComponent = m_DCGrid then
      DecisionGrid := nil
    else if AComponent = m_QR then
      QuickRep    := nil;
end;


procedure TDecisionGridRpt.ShowPreview;
begin
  if m_DCGrid = nil then Exit;

  m_nColCount   := m_DCGrid.ColCount;
  m_nFixedCols  := m_DCGrid.FixedCols;
  m_nRowCount   := m_DCGrid.RowCount;
  m_nFixedRows  := m_DCGrid.FixedRows;
  m_LabelList   := TList.Create;
  CreateLabels;
  m_QR.Preview;
  DeleteLabels;
end;

procedure TDecisionGridRpt.CreateLabels;
var
  nIndx    : Integer;
  DQRLabel : TQRLabel;
  nLeftPos : Integer;
begin
  m_QRDetailBand := nil;
  m_bBandCreated := False;

  if m_bCreateQR then
   begin
      m_QR := TQuickRep.Create(nil);
      m_QRDetailBand := TQRBand.Create(nil);
      m_QRDetailBand.Parent   	:= m_QR;
      m_QRDetailBand.BandType 	:= rbDetail;
      m_QR.OnNeedData         	:= m_QRNeedData;
      FQRBeforePrintEvent     	:= m_QR.BeforePrint;
      m_QR.BeforePrint 		:= m_QRBeforePrint;
      m_bBandCreated   		:= True;
   end
  else
   begin
      if m_QR.DataSet <> nil then
         m_QRDataset    := m_QR.DataSet;
      m_QR.DataSet := nil;
      if m_QR.Bands.HasDetail then
       begin
         m_QRDetailBand   := m_QR.Bands.DetailBand;
         m_bBandCreated   := False;
       end
      else
       begin
         m_QRDetailBand := TQRBand.Create(nil);
         m_QRDetailBand.Parent := m_QR;
         m_QRDetailBand.BandType := rbDetail;
         m_bBandCreated   := True;
       end;
       FQRBeforePrintEvent     := m_QR.BeforePrint;
      	m_QR.BeforePrint 	    := m_QRBeforePrint;
       m_QR.OnNeedData  		:= m_QRNeedData;
   end;

  m_nPrintableCols := (m_QRDetailBand.Width) div ColumnWidth;
  if m_nPrintableCols >= m_nColCount then
     m_nPrintableCols := m_nColCount-1
  else if m_bOutOfPageCaution then
     MessageDlg('Visible columns width exceed page width.'+#13#10+'Try changing Grid column width or page width.',mtWarning,[mbOk],0);


  for nIndx := 0 to m_LabelList.Count-1 do
     TQRLabel(m_LabelList.Items[nIndx]).Free;
  m_LabelList.Clear;
  m_QRDetailBand.Height := RowHeight;
  nLeftPos := 0;
  for nIndx := 0 to m_nPrintableCols-1 do
   begin
     DQRLabel          := TQRLabel.Create(nil);
     DQRLabel.Parent   := m_QRDetailBand;
     DQRLabel.Alignment:= taCenter;
     DQRLabel.Caption  := '';
     DQRLabel.Transparent := False;
     DQRLabel.Top      := 0;
     DQRLabel.Height   := RowHeight;
     DQRLabel.Left     := nLeftPos;
     DQRLabel.AutoSize := False;
     DQRLabel.Width    := ColumnWidth;
     m_LabelList.Add(DQRLabel);
     nLeftPos := nLeftPos + ColumnWidth;
   end;
  m_nCurrentRow := -m_nFixedRows;
end;


procedure TDecisionGridRpt.DeleteLabels;
var
  nIndx : Integer;
begin
  for nIndx := 0 to m_LabelList.Count-1 do
        TQRLabel(m_LabelList.Items[nIndx]).Free;
  m_LabelList.Clear;
  m_LabelList.Free;

  if m_bCreateQR then
   begin
      m_QRDetailBand.Free;
      m_QR.Free;
      m_QRDetailBand := nil;
      m_QR := nil;
   end
  else
   begin
      m_QR.DataSet := m_QRDataset;
      m_QR.BeforePrint := FQRBeforePrintEvent;
      if m_bBandCreated then
       begin
         m_QRDetailBand.Free;
         m_QRDetailBand := nil;
       end
      else
       begin
         m_QRDetailBand := nil;
       end;
   end;
end;


procedure TDecisionGridRpt.m_QRNeedData(Sender: TObject;
  var MoreData: Boolean);
var
  nIndx : Integer;
  aLabel : TQRLabel;
  sTemp  : String;
  DQState:TDecisionDrawState;

      procedure DrawFixedRows;
      begin
          if (dsRowCaption in DQState) or  (dsColCaption in DQState) then
           begin
             aLabel.Color := CaptionColor;
             aLabel.Font.Assign(CaptionFont);
           end
          else if (dsRowValue in DQState) or (dsColValue in DQState) then
           begin
              if dsSum in  DQState then
               begin
                aLabel.Color := LabelSumColor;
                aLabel.Font.Assign(LabelFont);
               end
              else
               begin
                aLabel.Color := LabelColor;
                aLabel.Font.Assign(LabelFont);
               end;
           end
          else if dsSum in  DQState then
           begin
             aLabel.Color := LabelSumColor;
             aLabel.Font.Assign(LabelFont);
           end
          else
           begin
             aLabel.Color := GridColor;
             aLabel.Font.Assign(LabelFont);
           end;
      end;

      procedure DrawGridCells;
      begin
           if (nIndx-m_nFixedCols+1) < 0 then
            begin
                   if (dsRowValue in DQState) or (dsColValue in DQState)  then
                     begin
                       if (dsSum in  DQState) then
                         begin
                            aLabel.Color := LabelSumColor;
                            aLabel.Font.Assign(LabelFont);
                         end
                       else
                        begin
                         aLabel.Color := LabelColor;
                         aLabel.Font.Assign(LabelFont);
                        end;
                     end
                   else
                     if  dsSum in  DQState then
                      begin
                        aLabel.Color := LabelSumColor;
                        aLabel.Font.Assign(LabelFont);
                      end
                     else
                      begin
                        aLabel.Color := LabelColor;
                        aLabel.Font.Assign(LabelFont);
                      end;
            end
          else
           begin
                  if dsSum in  DQState then
                   begin
                     aLabel.Color := DataSumColor;
                     aLabel.Font.Assign(DataFont);
                   end
                  else
                   begin
                     aLabel.Color := DataColor;
                     aLabel.Font.Assign(DataFont);
                   end;
           end;
      end;
begin
  if m_nCurrentRow <= (m_nRowCount-m_nFixedRows-1) then
   begin
     for nIndx := 0 to m_nPrintableCols-1 do
      begin
            aLabel := TQRLabel(m_LabelList.Items[nIndx]);
            aLabel.Caption := m_DCGrid.Cells[nIndx-m_nFixedCols+1,m_nCurrentRow];

            if ShowGridLines then
             begin
               aLabel.Frame.DrawRight  := True;
               aLabel.Frame.Drawbottom := True;
               aLabel.Frame.DrawTop    := True;
               aLabel.Frame.DrawLeft   := True;
             end;

            if aLabel.Caption = '[Error]' then
             begin
               aLabel.Caption := '';
             end;

            if PrintInColor then
            begin
                m_DCGrid.CellDrawState(nIndx-m_nFixedCols+1,m_nCurrentRow,sTemp,DQState);
                if m_nCurrentRow < 0 then
                 begin
                    DrawFixedRows;
                 end
                else
                  begin
                    DrawGridCells;
                  end;
             end;
      end;
     Inc(m_nCurrentRow);
     MoreData := True;
   end
  else
     MoreData := False;
end;

procedure TDecisionGridRpt.m_QRBeforePrint(Sender: TCustomQuickRep;
  var PrintReport: Boolean);
begin
  m_nCurrentRow := - m_nFixedRows;
end;

end.


